@@ -1,19 +1,31 @@
 import base64
+import email.message
+import email.policy
 import mimetypes
-import re
 from base64 import b64encode
 from collections.abc import Mapping, MutableMapping
 from copy import copy, deepcopy
+from email.charset import Charset
+from email.errors import HeaderParseError, InvalidHeaderDefect
+from email.header import decode_header
+from email.headerregistry import Address
 from email.mime.base import MIMEBase
 from email.utils import formatdate, getaddresses, parsedate_to_datetime, unquote
 from urllib.parse import urlsplit, urlunsplit
 
+import django.core.mail
 from django.conf import settings
-from django.core.mail.message import DEFAULT_ATTACHMENT_MIME_TYPE, sanitize_address
+from django.core.mail.message import DEFAULT_ATTACHMENT_MIME_TYPE
 from django.utils.encoding import force_str
-from django.utils.functional import Promise
+from django.utils.functional import Promise, cached_property
 from requests.structures import CaseInsensitiveDict
 
+try:
+    from django.core.mail.message import MIMEMixin
+except ImportError:
+    # Django >= 7.0
+    MIMEMixin = None
+
 from .exceptions import AnymailConfigurationError, AnymailInvalidAddress
 
 BASIC_NUMERIC_TYPES = (int, float)
@@ -22,6 +34,9 @@ BASIC_NUMERIC_TYPES = (int, float)
 UNSET = type("UNSET", (object,), {})  # Used as non-None default value
 
 
+SMTP7BIT = email.policy.SMTP.clone(cte_type="7bit")
+
+
 def concat_lists(*args):
     """
     Combines all non-UNSET args, by concatenating lists (or sequence-like types).
@@ -219,26 +234,21 @@ def parse_address_list(address_list, field=None):
     name_email_pairs = getaddresses(address_list_strings)
     if name_email_pairs == [] and address_list_strings == [""]:
         name_email_pairs = [("", "")]  # getaddresses ignores a single empty string
-    parsed = [
-        EmailAddress(display_name=name, addr_spec=email)
-        for (name, email) in name_email_pairs
-    ]
-
-    # Sanity-check, and raise useful errors
-    for address in parsed:
-        if address.username == "" or address.domain == "":
-            # Django SMTP allows username-only emails,
-            # but they're not meaningful with an ESP
-            errmsg = (
-                "Invalid email address '{problem}'" " parsed from '{source}'{where}."
-            ).format(
-                problem=address.addr_spec,
-                source=", ".join(address_list_strings),
-                where=" in `%s`" % field if field else "",
-            )
-            if len(parsed) > len(address_list):
-                errmsg += " (Maybe missing quotes around a display-name?)"
-            raise AnymailInvalidAddress(errmsg)
+    try:
+        parsed = [
+            EmailAddress(display_name=name, addr_spec=email)
+            for (name, email) in name_email_pairs
+        ]
+    except AnymailInvalidAddress as error:
+        # Add context to message.
+        msg = error.args[0]
+        if field:
+            msg += f" in `{field}`"
+        source = ", ".join(address_list_strings)
+        msg += f" parsed from {source!r}"
+        if any("," in value for value in address_list_strings):
+            msg += " (maybe missing quotes around a display-name?)"
+        raise error.__class__(msg, *error.args[1:]) from error
 
     return parsed
 
@@ -265,12 +275,10 @@ def parse_single_address(address, field=None):
         return parsed[0]
 
 
-class EmailAddress:
+class EmailAddress(Address):
     """A sanitized, complete email address with easy access
     to display-name, addr-spec (email), etc.
 
-    Similar to Python 3.6+ email.headerregistry.Address
-
     Instance properties, all read-only:
     :ivar str display_name:
         the address's display-name portion (unqouted, unescaped),
@@ -291,62 +299,137 @@ class EmailAddress:
         (also available as `str(EmailAddress)`)
     """
 
-    def __init__(self, display_name="", addr_spec=None):
-        self._address = None  # lazy formatted address
-        if addr_spec is None:
+    def __init__(self, display_name="", addr_spec=None, *, username="", domain=None):
+        if display_name is None:
+            display_name = ""
+
+        if addr_spec and not addr_spec.isascii():
+            # Work around python/cpython#81074 bug with Address(addr_spec=non_ascii).
             try:
-                display_name, addr_spec = display_name  # unpack (name,addr) tuple
-            except ValueError:
-                pass
+                username, domain = addr_spec.rsplit("@", 1)
+            except ValueError:  # not enough values to unpack
+                # local-part only address. Django supports that, but no Anymail ESP does.
+                raise AnymailInvalidAddress(
+                    f"Invalid email address {addr_spec!r}: missing @domain part"
+                ) from None
+            username = unquote_string(username)
+            addr_spec = None
 
-        # ESPs should clean or reject addresses containing newlines, but some
-        # extra protection can't hurt (and it seems to be a common oversight)
-        if "\n" in display_name or "\r" in display_name:
-            raise ValueError("EmailAddress display_name cannot contain newlines")
-        if "\n" in addr_spec or "\r" in addr_spec:
-            raise ValueError("EmailAddress addr_spec cannot contain newlines")
-
-        self.display_name = display_name
-        self.addr_spec = addr_spec
         try:
-            self.username, self.domain = addr_spec.split("@", 1)
-            # do we need to unquote username?
-        except ValueError:
-            self.username = addr_spec
-            self.domain = ""
+            super().__init__(display_name, username, domain, addr_spec)
+        except (HeaderParseError, IndexError, InvalidHeaderDefect, ValueError) as error:
+            msg = f"Invalid email address {addr_spec!r}"
+            if display_name:
+                msg += f" display_name={display_name!r}"
+            raise AnymailInvalidAddress(msg) from error
 
     def __repr__(self):
-        return "EmailAddress({display_name!r}, {addr_spec!r})".format(
-            display_name=self.display_name, addr_spec=self.addr_spec
-        )
+        return f"{self.__class__.__name__}({self.display_name!r}, {self.addr_spec!r})"
+
+    @cached_property
+    def address(self):
+        """
+        Fully-formatted string address, properly quoted and escaped.
+        Non-ASCII content is rendered as Unicode characters (not encoded).
+
+        :return str:
+        """
+        return self.format()
+
+    def format_display_name(self, *, use_rfc2047=False, use_quotes=False):
+        """
+        Return display_name formatted as specified.
+
+        :param bool | Literal["force"] use_rfc2047:
+            Whether to convert a non-ASCII display-name to an RFC 2047 encoded-word.
+            If "force", converts unconditionally.
+        :param bool | Literal["force"] use_quotes:
+            Whether to enclose display-name containing RFC 5322 special characters
+            in double quotes. If "force", quotes unconditionally.
+        :return str:
+        """
+        if use_rfc2047 == "force" or (use_rfc2047 and not self.display_name.isascii()):
+            return rfc2047_encode(self.display_name)
+        if use_quotes:
+            return quote_string(self.display_name, force=(use_quotes == "force"))
+        return self.display_name
+
+    def format_addr_spec(self, *, idna_encode=None):
+        """
+        Return addr_spec formatted as specified.
+
+        :param Callable[[str], str] | None idna_encode:
+            IDNA encoding function to apply to a non-ASCII domain.
+            If None, non-ASCII domains are left as-is.
+        :return str:
+        """
+        # (Note there is no 7-bit encoding for the username portion.
+        # If that local part isn't ASCII, probably the ESP will complain.)
+        if idna_encode and not self.domain.isascii():
+            return str(Address(username=self.username, domain=idna_encode(self.domain)))
+        return self.addr_spec
+
+    def format(self, *, use_rfc2047=False, idna_encode=None):
+        """
+        Return a fully-formatted email address.
+
+        Parameters control handling of non-ASCII address parts.
+
+        :param bool | Literal["force"] use_rfc2047:
+            Whether to convert a non-ASCII display-name to an RFC 2047 encoded-word.
+            Passing "force" applies rfc2047 unconditionally, even to ASCII-only names.
+        :param Callable[[str], str] | None idna_encode:
+            IDNA encoding function to apply to a non-ASCII domain.
+            If None, non-ASCII domains are left as-is.
+        :return str:
+        """
+        # This is essentially:
+        #     str(Address(display_name=self.format_display_name(...),
+        #                 addr_spec=self.format_addr_spec(...)))
+        # but working around around python/cpython#81074.
+        display_name = self.format_display_name(use_rfc2047=use_rfc2047)
+        username = self.username
+        if idna_encode and not self.domain.isascii():
+            domain = idna_encode(self.domain)
+        else:
+            domain = self.domain
+        return str(Address(display_name=display_name, username=username, domain=domain))
+
+    def as_dict(
+        self,
+        *,
+        name="name",
+        email="email",
+        use_rfc2047=False,
+        quote_name=False,
+        idna_encode=None,
+    ):
+        """
+        Return a dict representing the address, useful for JSON APIs.
+
+        :param str name: name of the display_name field
+        :param str email: name of the addr_spec field
+        :param bool | Literal["force"] use_rfc2047:
+            whether to encode the name field with rfc2047,
+            as in format_display_name()
+        :param bool | Literal["force"] quote_name:
+            whether to wrap the name field as an RFC 5322 quoted-string,
+            as in format_display_name()
+        :param Callable[[str], str] | None idna_encode:
+            IDNA encoding function to apply to a non-ASCII domain.
+            If None, non-ASCII domains are left as-is.
+        :return dict[str,str]:
+        """
+        obj = {email: self.format_addr_spec(idna_encode=idna_encode)}
+        if self.display_name:
+            obj[name] = self.format_display_name(
+                use_rfc2047=use_rfc2047, use_quotes=quote_name
+            )
+        return obj
 
     @property
-    def address(self):
-        if self._address is None:
-            # (you might be tempted to use `encoding=settings.DEFAULT_CHARSET` here,
-            # but that always forces the display-name to quoted-printable/base64,
-            # even when simple ascii would work fine--and be more readable)
-            self._address = self.formataddr()
-        return self._address
-
-    def formataddr(self, encoding=None):
-        """Return a fully-formatted email address, using encoding.
-
-        This is essentially the same as :func:`email.utils.formataddr`
-        on the EmailAddress's name and email properties, but uses
-        Django's :func:`~django.core.mail.message.sanitize_address`
-        for consistent handling of encoding (a.k.a. charset) and
-        proper handling of IDN domain portions.
-
-        :param str|None encoding:
-            the charset to use for the display-name portion;
-            default None uses ascii if possible, else 'utf-8'
-            (quoted-printable utf-8/base64)
-        """
-        sanitized = sanitize_address((self.display_name, self.addr_spec), encoding)
-        # sanitize_address() can introduce FWS with a long, non-ASCII display name.
-        # Must unfold it:
-        return re.sub(r"(\r|\n|\r\n)[ \t]", "", sanitized)
+    def uses_eai(self):
+        return not self.username.isascii()
 
     def __str__(self):
         return self.address
@@ -357,54 +440,116 @@ class Attachment:
 
     Normalized to have these properties:
     name: attachment filename; may be None
-    content: bytestream
-    mimetype: the content type; guessed if not explicit
-    inline: bool, True if attachment has a Content-ID header
+    content: bytes or str
+
+    mimetype: maintype/subtype (no params); guessed if not provided
+    maintype: content-type maintype
+    subtype: content-type subtype
+    charset: for text character encoding used (for text/* types)
+             default from settings.DEFAULT_CHARSET (which is utf-8 by default)
+    content_type: the full content-type, with charset param if text
+
+    inline: bool, True if Content-Disposition: inline
     content_id: for inline, the Content-ID (*with* <>); may be None
     cid: for inline, the Content-ID *without* <>; may be empty string
     """
 
-    def __init__(self, attachment, encoding):
+    def __init__(self, attachment):
         # Note that an attachment can be either a tuple of (filename, content, mimetype)
         # or a MIMEBase object. (Also, both filename and mimetype may be missing.)
         self._attachment = attachment
-        self.encoding = encoding  # or check attachment["Content-Encoding"] ???
-        self.inline = False
+
+        if isinstance(attachment, (MIMEBase, email.message.MIMEPart)):
+            # MIMEBase support is deprecated in Django 6.0, removed in 7.0.
+            # MIMEPart support is added in Django 6.0.
+            filename = attachment.get_filename()
+            content_type = attachment["Content-Type"]
+            mimetype = attachment.get_content_type()
+            charset = attachment.get_content_charset()
+            content = (
+                attachment.get_payload(decode=True)
+                if isinstance(attachment, MIMEBase)
+                else attachment.get_content()
+            )
+            if content is None:
+                content = attachment.as_bytes()
+            content_disposition = attachment.get_content_disposition()
+            inline = content_disposition == "inline" or (
+                content_disposition is None and "Content-ID" in attachment
+            )
+            content_id = attachment["Content-ID"]
+        else:
+            (_filename, _content, mimetype) = attachment
+            filename = force_non_lazy(_filename)
+            content = force_non_lazy(_content)
+            charset = None
+            inline = False
+            content_id = None
+
+            # Django supports both Django and Python EmailMessage as attachment content
+            if isinstance(content, django.core.mail.EmailMessage):
+                content = content.message()
+                if MIMEMixin is not None and isinstance(content, MIMEMixin):
+                    # Django < 6.0: can't use policy-based serialization below
+                    content = content.as_bytes(linesep="\r\n")
+                    if mimetype is None:
+                        mimetype = "message/rfc822"
+            if isinstance(content, (email.message.EmailMessage, email.message.Message)):
+                # Serialize attached message using conservative options
+                content = content.as_bytes(policy=SMTP7BIT)
+                if mimetype is None:
+                    mimetype = "message/rfc822"
+
+            content_type = mimetype
+
+        # Ensure mimetype
+        if mimetype is None and filename is not None:
+            # Guess missing mimetype from filename, borrowed from
+            # django.core.mail.EmailMessage._create_attachment()
+            try:
+                mimetype, _ = mimetypes.guess_type(filename)
+                content_type = None  # recreate from mimetype below
+            except TypeError:
+                pass
+        if mimetype is None:
+            mimetype = DEFAULT_ATTACHMENT_MIME_TYPE
+            content_type = None  # recreate from mimetype below
+
+        maintype, subtype = mimetype.split("/", 1)
+
+        # Ensure a charset for text/* types or any str content
+        if maintype == "text" or isinstance(content, str):
+            if charset is None:
+                charset = settings.DEFAULT_CHARSET
+            if content_type is None or charset not in content_type:
+                # Use MIMEPart to format a Content-Type header with charset
+                temp = email.message.MIMEPart()
+                temp.add_header("Content-Type", mimetype, charset=charset)
+                content_type = temp["Content-Type"]
+
+        # Ensure str for text/* types
+        if maintype == "text" and not isinstance(content, str):
+            content = content.decode(charset)
+
+        # Ensure content_type
+        if content_type is None:
+            content_type = mimetype
+
+        self.name = filename
+        self.content = content
+        self.mimetype = mimetype
+        self.maintype = maintype
+        self.subtype = subtype
+        self.charset = charset
+        self.content_type = content_type
+
+        self.inline = inline
         self.content_id = None
         self.cid = ""
-
-        if isinstance(attachment, MIMEBase):
-            self.name = attachment.get_filename()
-            self.content = attachment.get_payload(decode=True)
-            if self.content is None:
-                self.content = attachment.as_bytes()
-            self.mimetype = attachment.get_content_type()
-            # Content-Type includes charset if provided
-            self.content_type = attachment["Content-Type"]
-
-            content_disposition = attachment.get_content_disposition()
-            if content_disposition == "inline" or (
-                not content_disposition and "Content-ID" in attachment
-            ):
-                self.inline = True
-                self.content_id = attachment["Content-ID"]  # probably including <...>
-                if self.content_id is not None:
-                    self.cid = unquote(self.content_id)  # without the <, >
-        else:
-            (self.name, self.content, self.mimetype) = attachment
-            self.content_type = self.mimetype
-
-        self.name = force_non_lazy(self.name)
-        self.content = force_non_lazy(self.content)
-
-        # Guess missing mimetype from filename, borrowed from
-        # django.core.mail.EmailMessage._create_attachment()
-        if self.mimetype is None and self.name is not None:
-            self.mimetype, _ = mimetypes.guess_type(self.name)
-        if self.mimetype is None:
-            self.mimetype = DEFAULT_ATTACHMENT_MIME_TYPE
-        if self.content_type is None:
-            self.content_type = self.mimetype
+        if inline:
+            self.content_id = content_id
+            if content_id:
+                self.cid = unquote(content_id)  # without the <>
 
     def __repr__(self):
         details = [
@@ -421,12 +566,32 @@ class Attachment:
         return "Attachment<{details}>".format(details=", ".join(details))
 
     @property
-    def b64content(self):
-        """Content encoded as a base64 ascii string"""
+    def content_bytes(self):
+        """Content as bytes, using original charset"""
         content = self.content
         if isinstance(content, str):
-            content = content.encode(self.encoding)
-        return b64encode(content).decode("ascii")
+            content = content.encode(self.charset)
+        return content
+
+    @property
+    def content_utf8_bytes(self):
+        """Content as bytes, forcing utf-8 charset"""
+        if self.charset is None or self.charset == "utf-8":
+            return self.content_bytes
+        content = self.content
+        if isinstance(content, str):
+            content = content.encode("utf-8")
+        return content
+
+    @property
+    def b64content(self):
+        """Content encoded as a base64 7-bit string, using original charset"""
+        return b64encode(self.content_bytes).decode("ascii")
+
+    @property
+    def b64content_utf8(self):
+        """Content encoded as a base64 7-bit string, forcing utf-8 charset"""
+        return b64encode(self.content_utf8_bytes).decode("ascii")
 
 
 def get_anymail_setting(
@@ -543,6 +708,56 @@ def querydict_getfirst(qdict, field, default=UNSET):
         return qdict[field]  # raise appropriate KeyError
 
 
+def rfc2047_encode(text):
+    """Convert text to an RFC 2047 encoded-word using the utf-8 charset."""
+    # This uses Python's legacy email API, as the modern API does not expose
+    # a standalone RFC 2047 encoding operation.
+    return Charset("utf-8").header_encode(text)
+
+
+def rfc2047_decode(encoded):
+    """Convert an email header that may include RFC 2047 encoded-words to text."""
+    # This uses Python's legacy email API, as the modern API does not expose
+    # a standalone RFC 2047 decoding operation.
+    return "".join(
+        (
+            segment
+            if charset is None and isinstance(segment, str)
+            else segment.decode(charset or "ascii")
+        )
+        for segment, charset in decode_header(encoded)
+    )
+
+
+#: Set of characters that need special handling in a structured (address) header.
+RFC5322_SPECIALS = set('()<>@,:;."[]\\')
+
+
+def has_specials(text):
+    """Return True if text includes any RFC 5322 special characters."""
+    return not set(text).isdisjoint(RFC5322_SPECIALS)
+
+
+def quote_string(name, force=False):
+    """
+    Wrap name as an RFC 5322 quoted-string if necessary (or unconditionally if force).
+    """
+    # Adapted from email._header_value_parser.quote_string().
+    if force or has_specials(name):
+        escaped = str(name).replace("\\", "\\\\").replace('"', r"\"")
+        return f'"{escaped}"'
+    return name
+
+
+def unquote_string(name):
+    """
+    If name is an RFC 5322 quoted-string, return unquoted (and unescaped) value.
+    """
+    if name.startswith('"') and name.endswith('"'):
+        return name[1:-1].replace("\\\\", "\\").replace('\\"', '"')
+    return name
+
+
 def rfc2822date(dt):
     """Turn a datetime into a date string as specified in RFC 2822."""
     # This is almost the equivalent of Python's email.utils.format_datetime,
