@@ -1,24 +1,20 @@
 import json
 from datetime import datetime
 from email.encoders import encode_7or8bit
+from email.message import MIMEPart
 from email.mime.application import MIMEApplication
 from unittest.mock import ANY, patch
 
+import django
 from django.core import mail
-from django.core.mail import BadHeaderError
 from django.test import SimpleTestCase, override_settings, tag
 
 from anymail import __version__ as ANYMAIL_VERSION
 from anymail.exceptions import AnymailAPIError, AnymailUnsupportedFeature
 from anymail.inbound import AnymailInboundMessage
-from anymail.message import AnymailMessage, attach_inline_image_file
+from anymail.message import AnymailMessage, attach_inline_image
 
-from .utils import (
-    SAMPLE_IMAGE_FILENAME,
-    AnymailTestMixin,
-    sample_image_content,
-    sample_image_path,
-)
+from .utils import AnymailTestMixin, create_text_attachment, sample_image_content
 
 
 @tag("amazon_ses")
@@ -170,109 +166,88 @@ class AmazonSESBackendStandardEmailTests(AmazonSESBackendMockAPITestCase):
         self.assertEqual(
             params["Destination"],
             {
-                "ToAddresses": [
-                    "to1@example.com",
-                    '"Recipient, second" <to2@example.com>',
-                ],
-                "CcAddresses": ["cc1@example.com", "Also cc <cc2@example.com>"],
-                "BccAddresses": ["bcc1@example.com", "BCC 2 <bcc2@example.com>"],
+                "ToAddresses": ["to1@example.com", "to2@example.com"],
+                "CcAddresses": ["cc1@example.com", "cc2@example.com"],
+                "BccAddresses": ["bcc1@example.com", "bcc2@example.com"],
             },
         )
         # Bcc's shouldn't appear in the message itself:
         self.assertNotIn(b"bcc", params["Content"]["Raw"]["Data"])
 
     def test_non_ascii_headers(self):
-        self.message.subject = "Thử tin nhắn"  # utf-8 in subject header
-        self.message.to = ['"Người nhận" <to@example.com>']  # utf-8 in display name
-        self.message.cc = ["cc@thư.example.com"]  # utf-8 in domain
-        self.message.send()
+        # Amazon SES requires a flattened 7-bit message, including rfc2047
+        # encoded-words and IDNA encoding where appropriate.
+        # (SES does not support EAI non-ASCII local-part.)
+        email = mail.EmailMessage(
+            from_email='"Odesílatel, z adresy" <from@příklad.example.cz>',
+            to=['"Příjemce, na adresu" <to@příklad.example.cz>'],
+            cc=["cc@މިސާލު.example.mv"],
+            subject="Předmět e-mailu",
+            reply_to=['"Odpověď, adresa" <reply@příklad.example.cz>'],
+            headers={"X-Extra": "Další"},
+            body="Prostý text",
+        )
+        email.send()
+
         params = self.get_send_params()
         raw_mime = params["Content"]["Raw"]["Data"]
-        # Non-ASCII headers must use MIME encoded-word syntax:
-        self.assertIn(b"\nSubject: =?utf-8?b?VGjhu60gdGluIG5o4bqvbg==?=\n", raw_mime)
-        # Non-ASCII display names as well:
-        self.assertIn(
-            b"\nTo: =?utf-8?b?TmfGsOG7nWkgbmjhuq1u?= <to@example.com>\n", raw_mime
-        )
-        # Non-ASCII address domains must use Punycode:
-        self.assertIn(b"\nCc: cc@xn--th-e0a.example.com\n", raw_mime)
-        # SES doesn't support non-ASCII in the username@ part
-        # (RFC 6531 "SMTPUTF8" extension)
+        self.assertTrue(raw_mime.isascii(), "8bit Content.Raw.Data not allowed")
+        # Non-ASCII headers must use MIME encoded-word syntax.
+        # Python's email generator should handle this correctly, but spot check a few
+        # (slightly different in Django 5.2 with compat32 vs. 6.0 with policy.default)
+        self.assertIn(b"\nSubject: =?utf-8?b?UMWZZWRtxJt0", raw_mime)
+        self.assertIn(b"=?utf-8?q?Odes=C3=ADlatel=2C", raw_mime)
+        # Non-ASCII address domains must use IDNA
+        # (and it should be IDNA_ENCODER default idna2008):
+        self.assertIn(b"\nCc: cc@xn--qqbii6gdp.example.mv\n", raw_mime)
+        # Body must use a 7-bit encoding, probably quoted-printable
+        self.assertIn(b"\nProst=C3=BD=20text", raw_mime)
 
-        # Destinations must include all recipients (addr-spec only, must use Punycode):
+        # Destinations must include all recipients (addr-spec only, must use IDNA):
         self.assertEqual(
             params["Destination"],
             {
-                "ToAddresses": ["=?utf-8?b?TmfGsOG7nWkgbmjhuq1u?= <to@example.com>"],
-                "CcAddresses": ["cc@xn--th-e0a.example.com"],
+                "ToAddresses": ["to@xn--pklad-zsa96e.example.cz"],
+                "CcAddresses": ["cc@xn--qqbii6gdp.example.mv"],
             },
         )
 
     def test_attachments(self):
-        # These are \u2022 bullets ("\N{BULLET}") below:
-        text_content = "• Item one\n• Item two\n• Item three"
-        self.message.attach(
-            filename="Une pièce jointe.txt",  # utf-8 chars in filename
-            content=text_content,
-            mimetype="text/plain",
-        )
-
-        # Should guess mimetype if not provided...
-        png_content = b"PNG\xb4 pretend this is the contents of a png file"
-        self.message.attach(filename="test.png", content=png_content)
-
-        # Should work with a MIMEBase object (also tests no filename)...
-        pdf_content = b"PDF\xb4 pretend this is valid pdf params"
-        mimeattachment = MIMEApplication(pdf_content, "pdf")  # application/pdf
-        mimeattachment["Content-Disposition"] = "attachment"
-        self.message.attach(mimeattachment)
+        # Amazon SES accepts a raw MIME message and should handle anything
+        # valid generated by Django's EmailMessage.message().
+        # But Amazon SES requires 7-bit encoding throughout the message.
+        text_content = "pièce jointe\n"
+        self.message.attach(create_text_attachment(text_content, charset="iso-8859-1"))
+        self.message.attach("émoticône.img", b";-)", "image/x-emoticon")
+        image_data = sample_image_content()
+        cid = attach_inline_image(self.message, image_data, "test.png")
 
         self.message.send()
         sent_message = self.get_sent_message()
         attachments = sent_message.attachments
-        self.assertEqual(len(attachments), 3)
+        self.assertEqual(len(attachments), 2)
 
         self.assertEqual(attachments[0].get_content_type(), "text/plain")
-        self.assertEqual(attachments[0].get_filename(), "Une pièce jointe.txt")
-        self.assertEqual(attachments[0].get_param("charset"), "utf-8")
+        self.assertIsNone(attachments[0].get_filename())  # no filename specified
+        self.assertEqual(attachments[0].get_content_charset(), "iso-8859-1")
         self.assertEqual(attachments[0].get_content_text(), text_content)
 
-        self.assertEqual(attachments[1].get_content_type(), "image/png")
+        self.assertEqual(attachments[1].get_content_type(), "image/x-emoticon")
         # not inline:
         self.assertEqual(attachments[1].get_content_disposition(), "attachment")
-        self.assertEqual(attachments[1].get_filename(), "test.png")
-        self.assertEqual(attachments[1].get_content_bytes(), png_content)
-
-        self.assertEqual(attachments[2].get_content_type(), "application/pdf")
-        self.assertIsNone(attachments[2].get_filename())  # no filename specified
-        self.assertEqual(attachments[2].get_content_bytes(), pdf_content)
-
-    def test_embedded_images(self):
-        image_filename = SAMPLE_IMAGE_FILENAME
-        image_path = sample_image_path(image_filename)
-        image_data = sample_image_content(image_filename)
-
-        cid = attach_inline_image_file(self.message, image_path, domain="example.com")
-        html_content = (
-            '<p>This has an <img src="cid:%s" alt="inline" /> image.</p>' % cid
-        )
-        self.message.attach_alternative(html_content, "text/html")
-
-        self.message.send()
-        sent_message = self.get_sent_message()
-
-        self.assertEqual(sent_message.html, html_content)
+        self.assertEqual(attachments[1].get_filename(), "émoticône.img")
+        self.assertEqual(attachments[1].get_content_bytes(), b";-)")
 
         inlines = sent_message.content_id_map
         self.assertEqual(len(inlines), 1)
         self.assertEqual(inlines[cid].get_content_type(), "image/png")
-        self.assertEqual(inlines[cid].get_filename(), image_filename)
+        self.assertEqual(inlines[cid].get_filename(), "test.png")
         self.assertEqual(inlines[cid].get_content_bytes(), image_data)
 
-        # Make sure neither the html nor the inline image is treated as an attachment:
-        params = self.get_send_params()
+        # Make sure it used 7-bit encoding
+        params = self.get_send_params(operation_name="send_email")
         raw_mime = params["Content"]["Raw"]["Data"]
-        self.assertNotIn(b"\nContent-Disposition: attachment", raw_mime)
+        self.assertTrue(raw_mime.isascii(), "8bit Content.Raw.Data not allowed")
 
     def test_multiple_html_alternatives(self):
         # Multiple alternatives *are* allowed
@@ -339,7 +314,17 @@ class AmazonSESBackendStandardEmailTests(AmazonSESBackendMockAPITestCase):
         self.message.attach("sample.csv", "Это attachment", "text/csv")
 
         # Also force a CTE 8bit attachment (which normally defaults to CTE base64):
-        att = MIMEApplication("Это data".encode("utf8"), "data", encode_7or8bit)
+        if django.VERSION >= (6, 0):
+            att = MIMEPart()
+            att.set_content(
+                "Это data".encode("utf8"),
+                maintype="application",
+                subtype="data",
+                cte="8bit",
+            )
+        else:
+            # MIMEBase attachments deprecated in Django 6.0, removed in Django 7.0
+            att = MIMEApplication("Это data".encode("utf8"), "data", encode_7or8bit)
         self.assertEqual(att["Content-Transfer-Encoding"], "8bit")
         self.message.attach(att)
 
@@ -446,25 +431,25 @@ class AmazonSESBackendStandardEmailTests(AmazonSESBackendMockAPITestCase):
         # Since we build the raw MIME message, we're responsible for preventing header
         # injection. django.core.mail.EmailMessage.message() implements most of that
         # (for the SMTP backend); spot check some likely cases just to be sure...
-        with self.assertRaises(BadHeaderError):
+        with self.assertRaises(ValueError):
             mail.send_mail(
                 "Subject\r\ninjected", "Body", "from@example.com", ["to@example.com"]
             )
-        with self.assertRaises(BadHeaderError):
+        with self.assertRaises(ValueError):
             mail.send_mail(
                 "Subject",
                 "Body",
                 '"Display-Name\nInjected" <from@example.com>',
                 ["to@example.com"],
             )
-        with self.assertRaises(BadHeaderError):
+        with self.assertRaises(ValueError):
             mail.send_mail(
                 "Subject",
                 "Body",
                 "from@example.com",
                 ['"Display-Name\rInjected" <to@example.com>'],
             )
-        with self.assertRaises(BadHeaderError):
+        with self.assertRaises(ValueError):
             mail.EmailMessage(
                 "Subject",
                 "Body",
@@ -497,7 +482,7 @@ class AmazonSESBackendAnymailFeatureTests(AmazonSESBackendMockAPITestCase):
         raw_mime = params["Content"]["Raw"]["Data"]
         self.assertEqual(
             params["Destination"],
-            {"ToAddresses": ["Envelope <envelope-to@example.com>"]},
+            {"ToAddresses": ["envelope-to@example.com"]},
         )
         self.assertIn(b"\nTo: Spoofed <spoofed-to@elsewhere.example.org>\n", raw_mime)
         self.assertNotIn(b"envelope-to@example.com", raw_mime)
