@@ -1,9 +1,6 @@
 import json
-from base64 import b64decode, b64encode
 from datetime import date, datetime, timezone
 from decimal import Decimal
-from email.mime.base import MIMEBase
-from email.mime.image import MIMEImage
 
 from django.core import mail
 from django.test import SimpleTestCase, override_settings, tag
@@ -18,17 +15,17 @@ from anymail.exceptions import (
     AnymailSerializationError,
     AnymailUnsupportedFeature,
 )
-from anymail.message import AnymailMessage, attach_inline_image_file
+from anymail.message import AnymailMessage, attach_inline_image
 
 from .mock_requests_backend import (
     RequestsBackendMockAPITestCase,
     SessionSharingTestCases,
 )
 from .utils import (
-    SAMPLE_IMAGE_FILENAME,
     AnymailTestMixin,
+    create_text_attachment,
+    decode_att,
     sample_image_content,
-    sample_image_path,
 )
 
 
@@ -192,6 +189,13 @@ class BrevoBackendStandardEmailTests(BrevoBackendMockAPITestCase):
         with self.assertRaisesMessage(AnymailSerializationError, "Decimal"):
             self.message.send()
 
+    def test_extra_headers_non_ascii(self):
+        self.message.extra_headers = {"X-Extra": "Další"}
+        with self.assertRaisesMessage(
+            AnymailUnsupportedFeature, "non-ASCII characters in 'X-Extra' header"
+        ):
+            self.message.send()
+
     def test_reply_to(self):
         self.message.reply_to = ['"Reply recipient" <reply@example.com']
         self.message.send()
@@ -222,101 +226,118 @@ class BrevoBackendStandardEmailTests(BrevoBackendMockAPITestCase):
             data["replyTo"], {"name": "Reply recipient", "email": "reply@example.com"}
         )
 
-    def test_attachments(self):
-        text_content = "* Item one\n* Item two\n* Item three"
-        self.message.attach(
-            filename="test.txt", content=text_content, mimetype="text/plain"
+    def test_non_ascii_headers(self):
+        # Brevo correctly encodes non-ASCII display-names -- unless they contain commas
+        # (see next test). It requires IDNA encoding for non-ASCII domain names.
+        # It correctly encodes non-ASCII subjects, but sends raw utf-8 for other headers.
+        # Brevo supports EAI in all address fields (with some bugs--see docs).
+        email = mail.EmailMessage(
+            from_email='"Odesílatel z adresy" <from-тест@příklad.example.cz>',
+            to=['"Příjemce na adresu" <to-тест@příklad.example.cz>'],
+            subject="Předmět e-mailu",
+            reply_to=['"Odpověď adresa" <reply-тест@příklad.example.cz>'],  # no comma
+            # headers={"X-Extra": "Další"},  # not supported
+            body="Prostý text",
+        )
+        email.send()
+        data = self.get_api_call_json()
+        self.assertEqual(
+            data["sender"],
+            {
+                "name": "Odesílatel z adresy",
+                "email": "from-тест@xn--pklad-zsa96e.example.cz",
+            },
+        )
+        self.assertEqual(
+            data["to"],
+            [
+                {
+                    "name": "Příjemce na adresu",
+                    "email": "to-тест@xn--pklad-zsa96e.example.cz",
+                }
+            ],
+        )
+        self.assertEqual(data["subject"], "Předmět e-mailu")
+        self.assertEqual(
+            data["replyTo"],
+            {
+                "name": "Odpověď adresa",
+                "email": "reply-тест@xn--pklad-zsa96e.example.cz",
+            },
         )
 
-        # Should guess mimetype if not provided...
-        png_content = b"PNG\xb4 pretend this is the contents of a png file"
-        self.message.attach(filename="test.png", content=png_content)
+    def test_non_ascii_display_names_with_commas(self):
+        # Workaround Brevo bug that drops non-ASCII display-names with special chars
+        # by using rfc2047 encoded-words. (But don't use this in Reply-To, where Brevo
+        # incorrectly puts the encoded-word inside a quoted-string.)
+        email = mail.EmailMessage(
+            from_email='"Odesílatel, z adresy" <from@příklad.example.cz>',
+            to=['"Příjemce, na adresu" <to@příklad.example.cz>'],
+            reply_to=['"Odpověď, adresa" <reply@příklad.example.cz>'],
+        )
+        email.send()
+        data = self.get_api_call_json()
+        self.assertEqual(
+            data["sender"],
+            {
+                "name": "=?utf-8?q?Odes=C3=ADlatel=2C_z_adresy?=",
+                "email": "from@xn--pklad-zsa96e.example.cz",
+            },
+        )
+        self.assertEqual(
+            data["to"],
+            [
+                {
+                    "name": "=?utf-8?b?UMWZw61qZW1jZSwgbmEgYWRyZXN1?=",
+                    "email": "to@xn--pklad-zsa96e.example.cz",
+                }
+            ],
+        )
+        # Brevo transmits this as a raw utf8 header, which can cause problems.
+        # Using an rfc2047 encoded-word here prevents that, but gets wrapped
+        # in a quoted-string (unlike other address headers). That's the lesser
+        # of two bugs.
+        self.assertEqual(
+            data["replyTo"],
+            {
+                "name": "=?utf-8?b?T2Rwb3bEm8SPLCBhZHJlc2E=?=",
+                "email": "reply@xn--pklad-zsa96e.example.cz",
+            },
+        )
 
-        # Should work with a MIMEBase object (also tests no filename)...
-        pdf_content = b"PDF\xb4 pretend this is valid pdf data"
-        mimeattachment = MIMEBase("application", "pdf")
-        mimeattachment.set_payload(pdf_content)
-        self.message.attach(mimeattachment)
+    def test_attachments(self):
+        # Brevo guesses content type from the filename extension. It adds
+        # `charset=utf-8` to text content types, unconditionally.
+        # Brevo accepts non-ASCII filenames but incorrectly sends them
+        # as 8-bit utf-8 (without using RFC 2231 encoding).
+        # Brevo doesn't support inline images.
+        text_content = "pièce jointe\n"
+        self.message.attach(
+            create_text_attachment("pièce jointe\n", charset="iso-8859-1")
+        )
+        self.message.attach("émoticône.img", b";-)", "image/x-emoticon")
 
         self.message.send()
         data = self.get_api_call_json()
-        self.assertEqual(len(data["attachment"]), 3)
 
         attachments = data["attachment"]
+        self.assertEqual(len(attachments), 2)
+        # Anymail sends an empty filename if none specified, rather than
+        # trying to guess an extension. This will cause a Brevo API error.
+        self.assertEqual(attachments[0]["name"], "")  # no filename
+        # Text content *must* be utf-8 encoded.
         self.assertEqual(
-            attachments[0],
-            {
-                "name": "test.txt",
-                "content": b64encode(text_content.encode("utf-8")).decode("ascii"),
-            },
-        )
-        self.assertEqual(
-            attachments[1],
-            {"name": "test.png", "content": b64encode(png_content).decode("ascii")},
-        )
-        self.assertEqual(
-            attachments[2],
-            {"name": "", "content": b64encode(pdf_content).decode("ascii")},
-        )
-
-    def test_unicode_attachment_correctly_decoded(self):
-        self.message.attach(
-            "Une pièce jointe.html", "<p>\u2019</p>", mimetype="text/html"
-        )
-        self.message.send()
-        attachment = self.get_api_call_json()["attachment"][0]
-        self.assertEqual(attachment["name"], "Une pièce jointe.html")
-        self.assertEqual(
-            b64decode(attachment["content"]).decode("utf-8"), "<p>\u2019</p>"
+            decode_att(attachments[0]["content"]).decode("utf-8"), text_content
         )
+        self.assertEqual(attachments[1]["name"], "émoticône.img")
+        self.assertEqual(decode_att(attachments[1]["content"]), b";-)")
 
     def test_embedded_images(self):
-        # Brevo doesn't support inline image
-        # inline image are just added as a content attachment
-
-        image_filename = SAMPLE_IMAGE_FILENAME
-        image_path = sample_image_path(image_filename)
-
-        cid = attach_inline_image_file(self.message, image_path)  # Read from a png file
-        html_content = (
-            '<p>This has an <img src="cid:%s" alt="inline" /> image.</p>' % cid
-        )
-        self.message.attach_alternative(html_content, "text/html")
-
-        with self.assertRaises(AnymailUnsupportedFeature):
+        # Brevo doesn't support inline images
+        attach_inline_image(self.message, sample_image_content(), "test.png")
+        with self.assertRaisesMessage(AnymailUnsupportedFeature, "inline attachments"):
             self.message.send()
 
-    def test_attached_images(self):
-        image_filename = SAMPLE_IMAGE_FILENAME
-        image_path = sample_image_path(image_filename)
-        image_data = sample_image_content(image_filename)
-
-        # option 1: attach as a file
-        self.message.attach_file(image_path)
-
-        # option 2: construct the MIMEImage and attach it directly
-        image = MIMEImage(image_data)
-        self.message.attach(image)
-
-        self.message.send()
-
-        image_data_b64 = b64encode(image_data).decode("ascii")
-        data = self.get_api_call_json()
-        self.assertEqual(
-            data["attachment"][0],
-            {
-                "name": image_filename,  # the named one
-                "content": image_data_b64,
-            },
-        )
-        self.assertEqual(
-            data["attachment"][1],
-            {
-                "name": "",  # the unnamed one
-                "content": image_data_b64,
-            },
-        )
-
     def test_multiple_html_alternatives(self):
         self.message.body = "Text body"
         self.message.attach_alternative("<p>First html is OK</p>", "text/html")
@@ -391,6 +412,13 @@ class BrevoBackendAnymailFeatureTests(BrevoBackendMockAPITestCase):
         self.assertEqual(metadata["items"], 6)
         self.assertEqual(metadata["float"], 98.6)
 
+    def test_metadata_non_ascii(self):
+        self.message.metadata = {"user_id": "Další"}
+        with self.assertRaisesMessage(
+            AnymailUnsupportedFeature, "non-ASCII characters in metadata"
+        ):
+            self.message.send()
+
     def test_send_at(self):
         utc_plus_6 = get_fixed_timezone(6 * 60)
         utc_minus_8 = get_fixed_timezone(-8 * 60)
@@ -568,6 +596,18 @@ class BrevoBackendAnymailFeatureTests(BrevoBackendMockAPITestCase):
             {"notification_batch": "zx912"},
         )
 
+    def test_merge_metadata_non_ascii(self):
+        self.message.to = ["alice@example.com", "Bob <bob@example.com>"]
+        self.message.metadata = {"base": "ascii"}
+        self.message.merge_metadata = {
+            "alice@example.com": {"merge": "ascii"},
+            "bob@example.com": {"merge": "nøn-åscii"},
+        }
+        with self.assertRaisesMessage(
+            AnymailUnsupportedFeature, "non-ASCII characters in merge_metadata"
+        ):
+            self.message.send()
+
     def test_merge_headers(self):
         self.set_mock_response(json_data=self._mock_batch_response)
         self.message.to = ["alice@example.com", "Bob <bob@example.com>"]
@@ -605,6 +645,18 @@ class BrevoBackendAnymailFeatureTests(BrevoBackendMockAPITestCase):
             },
         )
 
+    def test_merge_headers_non_ascii(self):
+        self.message.to = ["alice@example.com", "Bob <bob@example.com>"]
+        self.message.extra_headers = {"X-Base": "ASCII"}
+        self.message.merge_headers = {
+            "alice@example.com": {"X-Merge": "ASCII"},
+            "bob@example.com": {"X-Merge": "nøn-åscii"},
+        }
+        with self.assertRaisesMessage(
+            AnymailUnsupportedFeature, "non-ASCII characters in merge_headers"
+        ):
+            self.message.send()
+
     def test_default_omits_options(self):
         """Make sure by default we don't send any ESP-specific options.
 
