@@ -1,8 +1,5 @@
 import json
-from base64 import b64encode
 from decimal import Decimal
-from email.mime.base import MIMEBase
-from email.mime.image import MIMEImage
 
 from django.core import mail
 from django.core.exceptions import ImproperlyConfigured
@@ -15,18 +12,17 @@ from anymail.exceptions import (
     AnymailSerializationError,
     AnymailUnsupportedFeature,
 )
-from anymail.message import AnymailMessage, attach_inline_image_file
+from anymail.message import AnymailMessage, attach_inline_image
 
 from .mock_requests_backend import (
     RequestsBackendMockAPITestCase,
     SessionSharingTestCases,
 )
 from .utils import (
-    SAMPLE_IMAGE_FILENAME,
     AnymailTestMixin,
+    create_text_attachment,
     decode_att,
     sample_image_content,
-    sample_image_path,
 )
 
 
@@ -202,117 +198,68 @@ class PostmarkBackendStandardEmailTests(PostmarkBackendMockAPITestCase):
         # don't lose other headers:
         self.assertEqual(data["Headers"], [{"Name": "X-Other", "Value": "Keep"}])
 
-    def test_attachments(self):
-        text_content = "* Item one\n* Item two\n* Item three"
-        self.message.attach(
-            filename="test.txt", content=text_content, mimetype="text/plain"
+    def test_non_ascii_headers(self):
+        # Postmark correctly encodes non-ASCII display-names and other headers
+        # (but requires IDNA encoding for non-ASCII domain names).
+        email = mail.EmailMessage(
+            from_email='"Odesílatel, z adresy" <from@příklad.example.cz>',
+            to=['"Příjemce, na adresu" <to@příklad.example.cz>'],
+            subject="Předmět e-mailu",
+            reply_to=['"Odpověď, adresa" <reply@příklad.example.cz>'],
+            headers={"X-Extra": "Další"},
+            body="Prostý text",
         )
+        email.send()
+        data = self.get_api_call_json()
+        self.assertEqual(
+            data["From"], '"Odesílatel, z adresy" <from@xn--pklad-zsa96e.example.cz>'
+        )
+        self.assertEqual(
+            data["To"], '"Příjemce, na adresu" <to@xn--pklad-zsa96e.example.cz>'
+        )
+        self.assertEqual(data["Subject"], "Předmět e-mailu")
+        self.assertEqual(
+            data["ReplyTo"], '"Odpověď, adresa" <reply@xn--pklad-zsa96e.example.cz>'
+        )
+        self.assertEqual(data["Headers"], [{"Name": "X-Extra", "Value": "Další"}])
 
-        # Should guess mimetype if not provided...
-        png_content = b"PNG\xb4 pretend this is the contents of a png file"
-        self.message.attach(filename="test.png", content=png_content)
-
-        # Should work with a MIMEBase object (also tests no filename)...
-        pdf_content = b"PDF\xb4 pretend this is valid pdf data"
-        mimeattachment = MIMEBase("application", "pdf")
-        mimeattachment.set_payload(pdf_content)
-        self.message.attach(mimeattachment)
+    def test_attachments(self):
+        # Postmark supports non-utf-8 content with charset in the `type` field.
+        # Postmark does not require attachment filenames. It allows non-ASCII
+        # filenames (though incorrectly applies rfc2047 when sending).
+        text_content = "pièce jointe\n"
+        self.message.attach(
+            create_text_attachment("pièce jointe\n", charset="iso-8859-1")
+        )
+        self.message.attach("émoticône.img", b";-)", "image/x-emoticon")
+        image_data = sample_image_content()
+        cid = attach_inline_image(self.message, image_data, "test.png")
 
         self.message.send()
         data = self.get_api_call_json()
+
         attachments = data["Attachments"]
         self.assertEqual(len(attachments), 3)
-        self.assertEqual(attachments[0]["Name"], "test.txt")
-        self.assertEqual(attachments[0]["ContentType"], "text/plain")
+        self.assertEqual(attachments[0]["Name"], "")  # no filename
         self.assertEqual(
-            decode_att(attachments[0]["Content"]).decode("ascii"), text_content
+            attachments[0]["ContentType"], 'text/plain; charset="iso-8859-1"'
+        )
+        self.assertEqual(
+            decode_att(attachments[0]["Content"]).decode("iso-8859-1"), text_content
         )
         self.assertNotIn("ContentID", attachments[0])
 
         # ContentType inferred from filename:
-        self.assertEqual(attachments[1]["ContentType"], "image/png")
-        self.assertEqual(attachments[1]["Name"], "test.png")
-        self.assertEqual(decode_att(attachments[1]["Content"]), png_content)
+        self.assertEqual(attachments[1]["ContentType"], "image/x-emoticon")
+        self.assertEqual(attachments[1]["Name"], "émoticône.img")
+        self.assertEqual(decode_att(attachments[1]["Content"]), b";-)")
         # make sure image not treated as inline:
         self.assertNotIn("ContentID", attachments[1])
 
-        self.assertEqual(attachments[2]["ContentType"], "application/pdf")
-        self.assertEqual(attachments[2]["Name"], "")  # none
-        self.assertEqual(decode_att(attachments[2]["Content"]), pdf_content)
-        self.assertNotIn("ContentID", attachments[2])
-
-    def test_unicode_attachment_correctly_decoded(self):
-        self.message.attach(
-            "Une pièce jointe.html", "<p>\u2019</p>", mimetype="text/html"
-        )
-        self.message.send()
-        data = self.get_api_call_json()
-        self.assertEqual(
-            data["Attachments"],
-            [
-                {
-                    "Name": "Une pièce jointe.html",
-                    "ContentType": "text/html",
-                    "Content": b64encode("<p>\u2019</p>".encode("utf-8")).decode(
-                        "ascii"
-                    ),
-                }
-            ],
-        )
-
-    def test_embedded_images(self):
-        image_filename = SAMPLE_IMAGE_FILENAME
-        image_path = sample_image_path(image_filename)
-        image_data = sample_image_content(image_filename)
-
-        cid = attach_inline_image_file(self.message, image_path)  # Read from a png file
-        html_content = (
-            '<p>This has an <img src="cid:%s" alt="inline" /> image.</p>' % cid
-        )
-        self.message.attach_alternative(html_content, "text/html")
-
-        self.message.send()
-        data = self.get_api_call_json()
-        self.assertEqual(data["HtmlBody"], html_content)
-
-        attachments = data["Attachments"]
-        self.assertEqual(len(attachments), 1)
-        self.assertEqual(attachments[0]["Name"], image_filename)
-        self.assertEqual(attachments[0]["ContentType"], "image/png")
-        self.assertEqual(decode_att(attachments[0]["Content"]), image_data)
-        self.assertEqual(attachments[0]["ContentID"], "cid:%s" % cid)
-
-    def test_attached_images(self):
-        image_filename = SAMPLE_IMAGE_FILENAME
-        image_path = sample_image_path(image_filename)
-        image_data = sample_image_content(image_filename)
-
-        # option 1: attach as a file
-        self.message.attach_file(image_path)
-
-        # option 2: construct the MIMEImage and attach it directly
-        image = MIMEImage(image_data)
-        self.message.attach(image)
-
-        image_data_b64 = b64encode(image_data).decode("ascii")
-
-        self.message.send()
-        data = self.get_api_call_json()
-        self.assertEqual(
-            data["Attachments"],
-            [
-                {
-                    "Name": image_filename,  # the named one
-                    "ContentType": "image/png",
-                    "Content": image_data_b64,
-                },
-                {
-                    "Name": "",  # the unnamed one
-                    "ContentType": "image/png",
-                    "Content": image_data_b64,
-                },
-            ],
-        )
+        self.assertEqual(attachments[2]["ContentType"], "image/png")
+        self.assertEqual(attachments[2]["Name"], "test.png")
+        self.assertEqual(decode_att(attachments[2]["Content"]), image_data)
+        self.assertEqual(attachments[2]["ContentID"], f"cid:{cid}")
 
     def test_multiple_html_alternatives(self):
         # Multiple alternatives not allowed
